// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.your_library.proto;

import "your_library_pseudo_playlist_config.proto";

option optimize_for = CODE_SIZE;

message YourLibraryContainsRequest {
    repeated string requested_uri = 3;
    YourLibraryPseudoPlaylistConfig pseudo_playlist_config = 4;
}
