// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`ListAccountRoles`](crate::operation::list_account_roles::builders::ListAccountRolesFluentBuilder) operation.
    /// This operation supports pagination; See [`into_paginator()`](crate::operation::list_account_roles::builders::ListAccountRolesFluentBuilder::into_paginator).
    ///
    /// - The fluent builder is configurable:
    ///   - [`next_token(impl Into<String>)`](crate::operation::list_account_roles::builders::ListAccountRolesFluentBuilder::next_token) / [`set_next_token(Option<String>)`](crate::operation::list_account_roles::builders::ListAccountRolesFluentBuilder::set_next_token):<br>required: **false**<br><p>The page token from the previous response output when you request subsequent pages.</p><br>
    ///   - [`max_results(i32)`](crate::operation::list_account_roles::builders::ListAccountRolesFluentBuilder::max_results) / [`set_max_results(Option<i32>)`](crate::operation::list_account_roles::builders::ListAccountRolesFluentBuilder::set_max_results):<br>required: **false**<br><p>The number of items that clients can request per page.</p><br>
    ///   - [`access_token(impl Into<String>)`](crate::operation::list_account_roles::builders::ListAccountRolesFluentBuilder::access_token) / [`set_access_token(Option<String>)`](crate::operation::list_account_roles::builders::ListAccountRolesFluentBuilder::set_access_token):<br>required: **true**<br><p>The token issued by the <code>CreateToken</code> API call. For more information, see <a href="https://docs.aws.amazon.com/singlesignon/latest/OIDCAPIReference/API_CreateToken.html">CreateToken</a> in the <i>IAM Identity Center OIDC API Reference Guide</i>.</p><br>
    ///   - [`account_id(impl Into<String>)`](crate::operation::list_account_roles::builders::ListAccountRolesFluentBuilder::account_id) / [`set_account_id(Option<String>)`](crate::operation::list_account_roles::builders::ListAccountRolesFluentBuilder::set_account_id):<br>required: **true**<br><p>The identifier for the AWS account that is assigned to the user.</p><br>
    /// - On success, responds with [`ListAccountRolesOutput`](crate::operation::list_account_roles::ListAccountRolesOutput) with field(s):
    ///   - [`next_token(Option<String>)`](crate::operation::list_account_roles::ListAccountRolesOutput::next_token): <p>The page token client that is used to retrieve the list of accounts.</p>
    ///   - [`role_list(Option<Vec::<RoleInfo>>)`](crate::operation::list_account_roles::ListAccountRolesOutput::role_list): <p>A paginated response with the list of roles and the next token if more results are available.</p>
    /// - On failure, responds with [`SdkError<ListAccountRolesError>`](crate::operation::list_account_roles::ListAccountRolesError)
    pub fn list_account_roles(&self) -> crate::operation::list_account_roles::builders::ListAccountRolesFluentBuilder {
        crate::operation::list_account_roles::builders::ListAccountRolesFluentBuilder::new(self.handle.clone())
    }
}
