// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto2";

package spotify.event_sender.proto;

option optimize_for = CODE_SIZE;

message AdDecisionEvent {
    optional string request_id = 1;
    optional string decision_request_id = 2;
    optional string decision_type = 3;
}
