// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto3";

package spotify.player.esperanto.proto;

import "es_command_options.proto";
import "es_logging_params.proto";
import "es_play_options.proto";
import "es_prepare_play.proto";

option objc_class_prefix = "ESP";
option optimize_for = CODE_SIZE;
option java_package = "com.spotify.player.esperanto.proto";

message PlayRequest {
    PreparePlayRequest prepare_play_request = 1;
    PlayOptions play_options = 2;
    CommandOptions options = 3;
    LoggingParams logging_params = 4;
}

message PlayPreparedRequest {
    string session_id = 1;
    PlayOptions play_options = 2;
    CommandOptions options = 3;
    LoggingParams logging_params = 4;
}
