// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`ListStreams`](crate::operation::list_streams::builders::ListStreamsFluentBuilder) operation.
    /// This operation supports pagination; See [`into_paginator()`](crate::operation::list_streams::builders::ListStreamsFluentBuilder::into_paginator).
    ///
    /// - The fluent builder is configurable:
    ///   - [`max_results(i32)`](crate::operation::list_streams::builders::ListStreamsFluentBuilder::max_results) / [`set_max_results(Option<i32>)`](crate::operation::list_streams::builders::ListStreamsFluentBuilder::set_max_results):<br>required: **false**<br><p>The maximum number of streams to return in the response. The default is 10,000.</p><br>
    ///   - [`next_token(impl Into<String>)`](crate::operation::list_streams::builders::ListStreamsFluentBuilder::next_token) / [`set_next_token(Option<String>)`](crate::operation::list_streams::builders::ListStreamsFluentBuilder::set_next_token):<br>required: **false**<br><p>If you specify this parameter, when the result of a <code>ListStreams</code> operation is truncated, the call returns the <code>NextToken</code> in the response. To get another batch of streams, provide this token in your next request.</p><br>
    ///   - [`stream_name_condition(StreamNameCondition)`](crate::operation::list_streams::builders::ListStreamsFluentBuilder::stream_name_condition) / [`set_stream_name_condition(Option<StreamNameCondition>)`](crate::operation::list_streams::builders::ListStreamsFluentBuilder::set_stream_name_condition):<br>required: **false**<br><p>Optional: Returns only streams that satisfy a specific condition. Currently, you can specify only the prefix of a stream name as a condition.</p><br>
    /// - On success, responds with [`ListStreamsOutput`](crate::operation::list_streams::ListStreamsOutput) with field(s):
    ///   - [`stream_info_list(Option<Vec::<StreamInfo>>)`](crate::operation::list_streams::ListStreamsOutput::stream_info_list): <p>An array of <code>StreamInfo</code> objects.</p>
    ///   - [`next_token(Option<String>)`](crate::operation::list_streams::ListStreamsOutput::next_token): <p>If the response is truncated, the call returns this element with a token. To get the next batch of streams, use this token in your next request.</p>
    /// - On failure, responds with [`SdkError<ListStreamsError>`](crate::operation::list_streams::ListStreamsError)
    pub fn list_streams(&self) -> crate::operation::list_streams::builders::ListStreamsFluentBuilder {
        crate::operation::list_streams::builders::ListStreamsFluentBuilder::new(self.handle.clone())
    }
}
