// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto2";

package spotify.on_demand_set_cosmos.proto;

option objc_class_prefix = "SPT";
option java_multiple_files = true;
option optimize_for = CODE_SIZE;
option java_package = "com.spotify.on_demand_set.proto";

message Response {
    optional bool success = 1;
}
