// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
#[allow(clippy::needless_question_mark)]
pub fn de_logging_enabled(
    decoder: &mut ::aws_smithy_xml::decode::ScopedDecoder,
) -> Result<crate::types::LoggingEnabled, ::aws_smithy_xml::decode::XmlDecodeError> {
    #[allow(unused_mut)]
    let mut builder = crate::types::LoggingEnabled::builder();
    while let Some(mut tag) = decoder.next_tag() {
        match tag.start_el() {
            s if s.matches("TargetBucket") /* TargetBucket com.amazonaws.s3#LoggingEnabled$TargetBucket */ =>  {
                let var_1 =
                    Some(
                        Result::<::std::string::String, ::aws_smithy_xml::decode::XmlDecodeError>::Ok(
                            ::aws_smithy_xml::decode::try_data(&mut tag)?.as_ref()
                            .into()
                        )
                        ?
                    )
                ;
                builder = builder.set_target_bucket(var_1);
            }
            ,
            s if s.matches("TargetGrants") /* TargetGrants com.amazonaws.s3#LoggingEnabled$TargetGrants */ =>  {
                let var_2 =
                    Some(
                        crate::protocol_serde::shape_target_grants::de_target_grants(&mut tag)
                        ?
                    )
                ;
                builder = builder.set_target_grants(var_2);
            }
            ,
            s if s.matches("TargetPrefix") /* TargetPrefix com.amazonaws.s3#LoggingEnabled$TargetPrefix */ =>  {
                let var_3 =
                    Some(
                        Result::<::std::string::String, ::aws_smithy_xml::decode::XmlDecodeError>::Ok(
                            ::aws_smithy_xml::decode::try_data(&mut tag)?.as_ref()
                            .into()
                        )
                        ?
                    )
                ;
                builder = builder.set_target_prefix(var_3);
            }
            ,
            s if s.matches("TargetObjectKeyFormat") /* TargetObjectKeyFormat com.amazonaws.s3#LoggingEnabled$TargetObjectKeyFormat */ =>  {
                let var_4 =
                    Some(
                        crate::protocol_serde::shape_target_object_key_format::de_target_object_key_format(&mut tag)
                        ?
                    )
                ;
                builder = builder.set_target_object_key_format(var_4);
            }
            ,
            _ => {}
        }
    }
    Ok(crate::serde_util::logging_enabled_correct_errors(builder)
        .build()
        .map_err(|_| ::aws_smithy_xml::decode::XmlDecodeError::custom("missing field"))?)
}

pub fn ser_logging_enabled(
    input: &crate::types::LoggingEnabled,
    writer: ::aws_smithy_xml::encode::ElWriter,
) -> Result<(), ::aws_smithy_types::error::operation::SerializationError> {
    #[allow(unused_mut)]
    let mut scope = writer.finish();
    {
        let mut inner_writer = scope.start_el("TargetBucket").finish();
        inner_writer.data(input.target_bucket.as_str());
    }
    if let Some(var_5) = &input.target_grants {
        let mut inner_writer = scope.start_el("TargetGrants").finish();
        for list_item_6 in var_5 {
            {
                let inner_writer = inner_writer.start_el("Grant");
                crate::protocol_serde::shape_target_grant::ser_target_grant(list_item_6, inner_writer)?
            }
        }
    }
    {
        let mut inner_writer = scope.start_el("TargetPrefix").finish();
        inner_writer.data(input.target_prefix.as_str());
    }
    if let Some(var_7) = &input.target_object_key_format {
        let inner_writer = scope.start_el("TargetObjectKeyFormat");
        crate::protocol_serde::shape_target_object_key_format::ser_target_object_key_format(var_7, inner_writer)?
    }
    scope.finish();
    Ok(())
}
