// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub(crate) fn type_erase_result<O, E>(
    result: ::std::result::Result<O, E>,
) -> ::std::result::Result<
    ::aws_smithy_runtime_api::client::interceptors::context::Output,
    ::aws_smithy_runtime_api::client::orchestrator::OrchestratorError<::aws_smithy_runtime_api::client::interceptors::context::Error>,
>
where
    O: ::std::fmt::Debug + ::std::marker::Send + ::std::marker::Sync + 'static,
    E: ::std::error::Error + std::fmt::Debug + ::std::marker::Send + ::std::marker::Sync + 'static,
{
    result
        .map(|output| ::aws_smithy_runtime_api::client::interceptors::context::Output::erase(output))
        .map_err(|error| ::aws_smithy_runtime_api::client::interceptors::context::Error::erase(error))
        .map_err(::std::convert::Into::into)
}

pub fn parse_http_error_metadata(
    _response_status: u16,
    response_headers: &::aws_smithy_runtime_api::http::Headers,
    response_body: &[u8],
) -> ::std::result::Result<::aws_smithy_types::error::metadata::Builder, ::aws_smithy_json::deserialize::error::DeserializeError> {
    crate::json_errors::parse_error_metadata(response_body, response_headers)
}

pub(crate) mod shape_create_token;

pub(crate) mod shape_create_token_with_iam;

pub(crate) mod shape_register_client;

pub(crate) mod shape_start_device_authorization;

pub(crate) fn or_empty_doc(data: &[u8]) -> &[u8] {
    if data.is_empty() {
        b"{}"
    } else {
        data
    }
}

pub(crate) mod shape_access_denied_exception;

pub(crate) mod shape_authorization_pending_exception;

pub(crate) mod shape_create_token_input;

pub(crate) mod shape_create_token_with_iam_input;

pub(crate) mod shape_expired_token_exception;

pub(crate) mod shape_internal_server_exception;

pub(crate) mod shape_invalid_client_exception;

pub(crate) mod shape_invalid_client_metadata_exception;

pub(crate) mod shape_invalid_grant_exception;

pub(crate) mod shape_invalid_redirect_uri_exception;

pub(crate) mod shape_invalid_request_exception;

pub(crate) mod shape_invalid_request_region_exception;

pub(crate) mod shape_invalid_scope_exception;

pub(crate) mod shape_register_client_input;

pub(crate) mod shape_slow_down_exception;

pub(crate) mod shape_start_device_authorization_input;

pub(crate) mod shape_unauthorized_client_exception;

pub(crate) mod shape_unsupported_grant_type_exception;

pub(crate) mod shape_scopes;
