// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.autodownload_esperanto.proto;

import "autodownload_config_common.proto";

option objc_class_prefix = "ESP";
option java_multiple_files = true;
option optimize_for = CODE_SIZE;
option java_package = "spotify.autodownload.esperanto.proto";

message AutoDownloadGetRequest {
    repeated string uri = 1;
}

message AutoDownloadGetResponse {
    AutoDownloadGlobalConfig global = 1;
    repeated AutoDownloadShowConfig show = 2;
    string error = 99;
}
