// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto2";

package spotify.event_sender.proto;

option optimize_for = CODE_SIZE;

message Prefetch {
    optional int64 strategies = 1;
    optional int64 strategy = 2;
    optional bytes file_id = 3;
    optional string track = 4;
    optional int64 prefetch_index = 5;
    optional int64 current_window_size = 6;
    optional int64 max_window_size = 7;
}
