// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.socialgraph_esperanto.proto;

import "socialgraph_response_status.proto";

option objc_class_prefix = "ESP";
option java_multiple_files = true;
option optimize_for = CODE_SIZE;
option java_package = "spotify.socialgraph.esperanto.proto";

message FollowRequest {
    repeated string username = 1;
    bool follow = 2;
}

message FollowResponse {
    ResponseStatus status = 1;
}
