// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
#[allow(clippy::needless_question_mark)]
pub fn de_inventory_encryption(
    decoder: &mut ::aws_smithy_xml::decode::ScopedDecoder,
) -> Result<crate::types::InventoryEncryption, ::aws_smithy_xml::decode::XmlDecodeError> {
    #[allow(unused_mut)]
    let mut builder = crate::types::InventoryEncryption::builder();
    while let Some(mut tag) = decoder.next_tag() {
        match tag.start_el() {
            s if s.matches("SSE-S3") /* SSES3 com.amazonaws.s3#InventoryEncryption$SSES3 */ =>  {
                let var_1 =
                    Some(
                        crate::protocol_serde::shape_sses3::de_sses3(&mut tag)
                        ?
                    )
                ;
                builder = builder.set_sses3(var_1);
            }
            ,
            s if s.matches("SSE-KMS") /* SSEKMS com.amazonaws.s3#InventoryEncryption$SSEKMS */ =>  {
                let var_2 =
                    Some(
                        crate::protocol_serde::shape_ssekms::de_ssekms(&mut tag)
                        ?
                    )
                ;
                builder = builder.set_ssekms(var_2);
            }
            ,
            _ => {}
        }
    }
    Ok(builder.build())
}

pub fn ser_inventory_encryption(
    input: &crate::types::InventoryEncryption,
    writer: ::aws_smithy_xml::encode::ElWriter,
) -> Result<(), ::aws_smithy_types::error::operation::SerializationError> {
    #[allow(unused_mut)]
    let mut scope = writer.finish();
    if let Some(_var_3) = &input.sses3 {
        scope.start_el("SSE-S3").finish();
    }
    if let Some(var_4) = &input.ssekms {
        let inner_writer = scope.start_el("SSE-KMS");
        crate::protocol_serde::shape_ssekms::ser_ssekms(var_4, inner_writer)?
    }
    scope.finish();
    Ok(())
}
