// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto3";

package spotify.stream_reporting_esperanto.proto;

import "status_code.proto";

option objc_class_prefix = "ESP";
option java_package = "com.spotify.stream_reporting_esperanto.proto";

message StatusResponse {
    StatusCode status_code = 1;
    string reason = 2;
}
