// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub fn ser_create_parallel_data_input_input(
    object: &mut ::aws_smithy_json::serialize::JsonObjectWriter,
    input: &crate::operation::create_parallel_data::CreateParallelDataInput,
) -> ::std::result::Result<(), ::aws_smithy_types::error::operation::SerializationError> {
    if let Some(var_1) = &input.name {
        object.key("Name").string(var_1.as_str());
    }
    if let Some(var_2) = &input.description {
        object.key("Description").string(var_2.as_str());
    }
    if let Some(var_3) = &input.parallel_data_config {
        #[allow(unused_mut)]
        let mut object_4 = object.key("ParallelDataConfig").start_object();
        crate::protocol_serde::shape_parallel_data_config::ser_parallel_data_config(&mut object_4, var_3)?;
        object_4.finish();
    }
    if let Some(var_5) = &input.encryption_key {
        #[allow(unused_mut)]
        let mut object_6 = object.key("EncryptionKey").start_object();
        crate::protocol_serde::shape_encryption_key::ser_encryption_key(&mut object_6, var_5)?;
        object_6.finish();
    }
    if let Some(var_7) = &input.client_token {
        object.key("ClientToken").string(var_7.as_str());
    }
    if let Some(var_8) = &input.tags {
        let mut array_9 = object.key("Tags").start_array();
        for item_10 in var_8 {
            {
                #[allow(unused_mut)]
                let mut object_11 = array_9.value().start_object();
                crate::protocol_serde::shape_tag::ser_tag(&mut object_11, item_10)?;
                object_11.finish();
            }
        }
        array_9.finish();
    }
    Ok(())
}
