// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto2";

package spotify.event_sender.proto;

option optimize_for = CODE_SIZE;

message ClientLocale {
    optional string client_default_locale = 1;
    optional string user_specified_locale = 2;
}
