// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto2";

package spotify.cosmos_util.proto;

option java_multiple_files = true;
option optimize_for = CODE_SIZE;
option java_package = "com.spotify.cosmos.util.proto";

message ArtistSyncState {
    optional string offline = 1;
    optional string inferred_offline = 2;
    optional uint32 sync_progress = 3;
}
