// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`DescribeStream`](crate::operation::describe_stream::builders::DescribeStreamFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`stream_name(impl Into<String>)`](crate::operation::describe_stream::builders::DescribeStreamFluentBuilder::stream_name) / [`set_stream_name(Option<String>)`](crate::operation::describe_stream::builders::DescribeStreamFluentBuilder::set_stream_name):<br>required: **false**<br><p>The name of the stream.</p><br>
    ///   - [`stream_arn(impl Into<String>)`](crate::operation::describe_stream::builders::DescribeStreamFluentBuilder::stream_arn) / [`set_stream_arn(Option<String>)`](crate::operation::describe_stream::builders::DescribeStreamFluentBuilder::set_stream_arn):<br>required: **false**<br><p>The Amazon Resource Name (ARN) of the stream.</p><br>
    /// - On success, responds with [`DescribeStreamOutput`](crate::operation::describe_stream::DescribeStreamOutput) with field(s):
    ///   - [`stream_info(Option<StreamInfo>)`](crate::operation::describe_stream::DescribeStreamOutput::stream_info): <p>An object that describes the stream.</p>
    /// - On failure, responds with [`SdkError<DescribeStreamError>`](crate::operation::describe_stream::DescribeStreamError)
    pub fn describe_stream(&self) -> crate::operation::describe_stream::builders::DescribeStreamFluentBuilder {
        crate::operation::describe_stream::builders::DescribeStreamFluentBuilder::new(self.handle.clone())
    }
}
