// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub fn ser_completed_multipart_upload(
    input: &crate::types::CompletedMultipartUpload,
    writer: ::aws_smithy_xml::encode::ElWriter,
) -> Result<(), ::aws_smithy_types::error::operation::SerializationError> {
    #[allow(unused_mut)]
    let mut scope = writer.finish();
    if let Some(var_1) = &input.parts {
        for list_item_2 in var_1 {
            {
                let inner_writer = scope.start_el("Part");
                crate::protocol_serde::shape_completed_part::ser_completed_part(list_item_2, inner_writer)?
            }
        }
    }
    scope.finish();
    Ok(())
}
