// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.stream_reporting_esperanto.proto;

option objc_class_prefix = "ESP";
option java_package = "com.spotify.stream_reporting_esperanto.proto";

enum MediaType {
    AUDIO = 0;
    VIDEO = 1;
}
