// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.profile_esperanto.proto.v1;

import "identity.proto";

option optimize_for = CODE_SIZE;

service ProfileService {
    rpc GetProfiles(GetProfilesRequest) returns (GetProfilesResponse);
    rpc SubscribeToProfiles(GetProfilesRequest) returns (stream GetProfilesResponse);
    rpc ChangeDisplayName(ChangeDisplayNameRequest) returns (ChangeDisplayNameResponse);
}

message GetProfilesRequest {
    repeated string usernames = 1;
}

message GetProfilesResponse {
    repeated identity.v3.UserProfile profiles = 1;
    int32 status_code = 2;
}

message ChangeDisplayNameRequest {
    string username = 1;
    string display_name = 2;
}

message ChangeDisplayNameResponse {
    int32 status_code = 1;
}
