// Copyright 2023 LiveKit, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package rpc;

option go_package = "github.com/livekit/livekit/pkg/service/rpc";

import "options.proto";
import "livekit_internal.proto";
import "livekit_rtc.proto";

service Signal {
  rpc RelaySignal(rpc.RelaySignalRequest) returns (rpc.RelaySignalResponse) {
    option (psrpc.options) = {
      stream: true
      topics: true
      topic_params: {
        names: ["node_id"]
        typed: true
        single_server: true
      };
    };
  };
}

message RelaySignalRequest {
  livekit.StartSession start_session = 1;
  repeated livekit.SignalRequest requests = 3;
  uint64 seq = 4;
  bool close = 5;
}

message RelaySignalResponse {
  repeated livekit.SignalResponse responses = 2;
  uint64 seq = 3;
  bool close = 4;
}
