// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
#[allow(missing_docs)] // documentation missing in model
#[non_exhaustive]
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::fmt::Debug)]
pub struct StopTextTranslationJobInput {
    /// <p>The job ID of the job to be stopped.</p>
    pub job_id: ::std::option::Option<::std::string::String>,
}
impl StopTextTranslationJobInput {
    /// <p>The job ID of the job to be stopped.</p>
    pub fn job_id(&self) -> ::std::option::Option<&str> {
        self.job_id.as_deref()
    }
}
impl StopTextTranslationJobInput {
    /// Creates a new builder-style object to manufacture [`StopTextTranslationJobInput`](crate::operation::stop_text_translation_job::StopTextTranslationJobInput).
    pub fn builder() -> crate::operation::stop_text_translation_job::builders::StopTextTranslationJobInputBuilder {
        crate::operation::stop_text_translation_job::builders::StopTextTranslationJobInputBuilder::default()
    }
}

/// A builder for [`StopTextTranslationJobInput`](crate::operation::stop_text_translation_job::StopTextTranslationJobInput).
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::default::Default, ::std::fmt::Debug)]
#[non_exhaustive]
pub struct StopTextTranslationJobInputBuilder {
    pub(crate) job_id: ::std::option::Option<::std::string::String>,
}
impl StopTextTranslationJobInputBuilder {
    /// <p>The job ID of the job to be stopped.</p>
    /// This field is required.
    pub fn job_id(mut self, input: impl ::std::convert::Into<::std::string::String>) -> Self {
        self.job_id = ::std::option::Option::Some(input.into());
        self
    }
    /// <p>The job ID of the job to be stopped.</p>
    pub fn set_job_id(mut self, input: ::std::option::Option<::std::string::String>) -> Self {
        self.job_id = input;
        self
    }
    /// <p>The job ID of the job to be stopped.</p>
    pub fn get_job_id(&self) -> &::std::option::Option<::std::string::String> {
        &self.job_id
    }
    /// Consumes the builder and constructs a [`StopTextTranslationJobInput`](crate::operation::stop_text_translation_job::StopTextTranslationJobInput).
    pub fn build(
        self,
    ) -> ::std::result::Result<
        crate::operation::stop_text_translation_job::StopTextTranslationJobInput,
        ::aws_smithy_types::error::operation::BuildError,
    > {
        ::std::result::Result::Ok(crate::operation::stop_text_translation_job::StopTextTranslationJobInput { job_id: self.job_id })
    }
}
