// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.narration_injection.proto;

import "tts-resolve.proto";

option optimize_for = CODE_SIZE;

service ClientTtsService {
    rpc GetTtsUrl(TtsRequest) returns (TtsResponse);
}

message TtsRequest {
    ResolveRequest.AudioFormat audio_format = 3;
    string language = 4;
    ResolveRequest.TtsVoice tts_voice = 5;
    ResolveRequest.TtsProvider tts_provider = 6;
    int32 sample_rate_hz = 7;
    
    oneof prompt {
        string text = 1;
        string ssml = 2;
    }
}

message TtsResponse {
    string url = 1;
}
