// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
#[allow(clippy::needless_question_mark)]
pub fn de_metadata_table_configuration_result(
    decoder: &mut ::aws_smithy_xml::decode::ScopedDecoder,
) -> Result<crate::types::MetadataTableConfigurationResult, ::aws_smithy_xml::decode::XmlDecodeError> {
    #[allow(unused_mut)]
    let mut builder = crate::types::MetadataTableConfigurationResult::builder();
    while let Some(mut tag) = decoder.next_tag() {
        match tag.start_el() {
            s if s.matches("S3TablesDestinationResult") /* S3TablesDestinationResult com.amazonaws.s3#MetadataTableConfigurationResult$S3TablesDestinationResult */ =>  {
                let var_1 =
                    Some(
                        crate::protocol_serde::shape_s3_tables_destination_result::de_s3_tables_destination_result(&mut tag)
                        ?
                    )
                ;
                builder = builder.set_s3_tables_destination_result(var_1);
            }
            ,
            _ => {}
        }
    }
    Ok(crate::serde_util::metadata_table_configuration_result_correct_errors(builder).build())
}
