// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto2";

package spotify.player.proto.state_restore;

option optimize_for = CODE_SIZE;

message TrackErrorRules {
    repeated string reasons = 1;
    required int32 num_attempted_tracks = 2;
    required int32 num_failed_tracks = 3;
}
