// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto2";

package spotify.player.proto;

option optimize_for = CODE_SIZE;

message MediaTypeNode {
    optional string current_uri = 1;
    optional string media_type = 2;
    optional string media_manifest_id = 3;
}
