// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto2";

package spotify.event_sender.proto;

option optimize_for = CODE_SIZE;

message DesktopAuthenticationFailureNonAuth {
    optional string action_hash = 1;
    optional string error_category = 2;
    optional int32 error_code = 3;
}
