// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`CreateParallelData`](crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`name(impl Into<String>)`](crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder::name) / [`set_name(Option<String>)`](crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder::set_name):<br>required: **true**<br><p>A custom name for the parallel data resource in Amazon Translate. You must assign a name that is unique in the account and region.</p><br>
    ///   - [`description(impl Into<String>)`](crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder::description) / [`set_description(Option<String>)`](crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder::set_description):<br>required: **false**<br><p>A custom description for the parallel data resource in Amazon Translate.</p><br>
    ///   - [`parallel_data_config(ParallelDataConfig)`](crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder::parallel_data_config) / [`set_parallel_data_config(Option<ParallelDataConfig>)`](crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder::set_parallel_data_config):<br>required: **true**<br><p>Specifies the format and S3 location of the parallel data input file.</p><br>
    ///   - [`encryption_key(EncryptionKey)`](crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder::encryption_key) / [`set_encryption_key(Option<EncryptionKey>)`](crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder::set_encryption_key):<br>required: **false**<br><p>The encryption key used to encrypt this object.</p><br>
    ///   - [`client_token(impl Into<String>)`](crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder::client_token) / [`set_client_token(Option<String>)`](crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder::set_client_token):<br>required: **true**<br><p>A unique identifier for the request. This token is automatically generated when you use Amazon Translate through an AWS SDK.</p><br>
    ///   - [`tags(Tag)`](crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder::tags) / [`set_tags(Option<Vec::<Tag>>)`](crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder::set_tags):<br>required: **false**<br><p>Tags to be associated with this resource. A tag is a key-value pair that adds metadata to a resource. Each tag key for the resource must be unique. For more information, see <a href="https://docs.aws.amazon.com/translate/latest/dg/tagging.html"> Tagging your resources</a>.</p><br>
    /// - On success, responds with [`CreateParallelDataOutput`](crate::operation::create_parallel_data::CreateParallelDataOutput) with field(s):
    ///   - [`name(Option<String>)`](crate::operation::create_parallel_data::CreateParallelDataOutput::name): <p>The custom name that you assigned to the parallel data resource.</p>
    ///   - [`status(Option<ParallelDataStatus>)`](crate::operation::create_parallel_data::CreateParallelDataOutput::status): <p>The status of the parallel data resource. When the resource is ready for you to use, the status is <code>ACTIVE</code>.</p>
    /// - On failure, responds with [`SdkError<CreateParallelDataError>`](crate::operation::create_parallel_data::CreateParallelDataError)
    pub fn create_parallel_data(&self) -> crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder {
        crate::operation::create_parallel_data::builders::CreateParallelDataFluentBuilder::new(self.handle.clone())
    }
}
