// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub fn ser_glacier_job_parameters(
    input: &crate::types::GlacierJobParameters,
    writer: ::aws_smithy_xml::encode::ElWriter,
) -> Result<(), ::aws_smithy_types::error::operation::SerializationError> {
    #[allow(unused_mut)]
    let mut scope = writer.finish();
    {
        let mut inner_writer = scope.start_el("Tier").finish();
        inner_writer.data(input.tier.as_str());
    }
    scope.finish();
    Ok(())
}
