// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
#[allow(missing_docs)] // documentation missing in model
#[non_exhaustive]
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::fmt::Debug)]
pub struct ListLexiconsInput {
    /// <p>An opaque pagination token returned from previous <code>ListLexicons</code> operation. If present, indicates where to continue the list of lexicons.</p>
    pub next_token: ::std::option::Option<::std::string::String>,
}
impl ListLexiconsInput {
    /// <p>An opaque pagination token returned from previous <code>ListLexicons</code> operation. If present, indicates where to continue the list of lexicons.</p>
    pub fn next_token(&self) -> ::std::option::Option<&str> {
        self.next_token.as_deref()
    }
}
impl ListLexiconsInput {
    /// Creates a new builder-style object to manufacture [`ListLexiconsInput`](crate::operation::list_lexicons::ListLexiconsInput).
    pub fn builder() -> crate::operation::list_lexicons::builders::ListLexiconsInputBuilder {
        crate::operation::list_lexicons::builders::ListLexiconsInputBuilder::default()
    }
}

/// A builder for [`ListLexiconsInput`](crate::operation::list_lexicons::ListLexiconsInput).
#[derive(::std::clone::Clone, ::std::cmp::PartialEq, ::std::default::Default, ::std::fmt::Debug)]
#[non_exhaustive]
pub struct ListLexiconsInputBuilder {
    pub(crate) next_token: ::std::option::Option<::std::string::String>,
}
impl ListLexiconsInputBuilder {
    /// <p>An opaque pagination token returned from previous <code>ListLexicons</code> operation. If present, indicates where to continue the list of lexicons.</p>
    pub fn next_token(mut self, input: impl ::std::convert::Into<::std::string::String>) -> Self {
        self.next_token = ::std::option::Option::Some(input.into());
        self
    }
    /// <p>An opaque pagination token returned from previous <code>ListLexicons</code> operation. If present, indicates where to continue the list of lexicons.</p>
    pub fn set_next_token(mut self, input: ::std::option::Option<::std::string::String>) -> Self {
        self.next_token = input;
        self
    }
    /// <p>An opaque pagination token returned from previous <code>ListLexicons</code> operation. If present, indicates where to continue the list of lexicons.</p>
    pub fn get_next_token(&self) -> &::std::option::Option<::std::string::String> {
        &self.next_token
    }
    /// Consumes the builder and constructs a [`ListLexiconsInput`](crate::operation::list_lexicons::ListLexiconsInput).
    pub fn build(
        self,
    ) -> ::std::result::Result<crate::operation::list_lexicons::ListLexiconsInput, ::aws_smithy_types::error::operation::BuildError> {
        ::std::result::Result::Ok(crate::operation::list_lexicons::ListLexiconsInput { next_token: self.next_token })
    }
}
