/*
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0
 */

/// Classifiers that can inspect a response and determine if it should be retried.
pub mod classifiers;
