// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.stream_reporting_esperanto.proto;

import "stream_handle.proto";

option objc_class_prefix = "ESP";
option java_package = "com.spotify.stream_reporting_esperanto.proto";

message StreamSeekRequest {
    StreamHandle stream_handle = 1;
    uint64 from_position = 3;
    uint64 to_position = 4;
}
