// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.player.esperanto.proto;

option objc_class_prefix = "ESP";
option optimize_for = CODE_SIZE;
option java_package = "com.spotify.player.esperanto.proto";

message CommandOptions {
    bool override_restrictions = 1;
    bool only_for_local_device = 2;
    bool system_initiated = 3;
    bytes only_for_playback_id = 4;
}
