// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`GetPublicAccessBlock`](crate::operation::get_public_access_block::builders::GetPublicAccessBlockFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`bucket(impl Into<String>)`](crate::operation::get_public_access_block::builders::GetPublicAccessBlockFluentBuilder::bucket) / [`set_bucket(Option<String>)`](crate::operation::get_public_access_block::builders::GetPublicAccessBlockFluentBuilder::set_bucket):<br>required: **true**<br><p>The name of the Amazon S3 bucket whose <code>PublicAccessBlock</code> configuration you want to retrieve.</p><br>
    ///   - [`expected_bucket_owner(impl Into<String>)`](crate::operation::get_public_access_block::builders::GetPublicAccessBlockFluentBuilder::expected_bucket_owner) / [`set_expected_bucket_owner(Option<String>)`](crate::operation::get_public_access_block::builders::GetPublicAccessBlockFluentBuilder::set_expected_bucket_owner):<br>required: **false**<br><p>The account ID of the expected bucket owner. If the account ID that you provide does not match the actual owner of the bucket, the request fails with the HTTP status code <code>403 Forbidden</code> (access denied).</p><br>
    /// - On success, responds with [`GetPublicAccessBlockOutput`](crate::operation::get_public_access_block::GetPublicAccessBlockOutput) with field(s):
    ///   - [`public_access_block_configuration(Option<PublicAccessBlockConfiguration>)`](crate::operation::get_public_access_block::GetPublicAccessBlockOutput::public_access_block_configuration): <p>The <code>PublicAccessBlock</code> configuration currently in effect for this Amazon S3 bucket.</p>
    /// - On failure, responds with [`SdkError<GetPublicAccessBlockError>`](crate::operation::get_public_access_block::GetPublicAccessBlockError)
    pub fn get_public_access_block(&self) -> crate::operation::get_public_access_block::builders::GetPublicAccessBlockFluentBuilder {
        crate::operation::get_public_access_block::builders::GetPublicAccessBlockFluentBuilder::new(self.handle.clone())
    }
}
