// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto2";

package spotify.event_sender.proto;

option optimize_for = CODE_SIZE;

message ConnectPullFailure {
    optional bytes transfer_data = 1;
    optional int64 error_code = 2;
    map<string, string> reasons = 3;
}
