// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`GetBucketRequestPayment`](crate::operation::get_bucket_request_payment::builders::GetBucketRequestPaymentFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`bucket(impl Into<String>)`](crate::operation::get_bucket_request_payment::builders::GetBucketRequestPaymentFluentBuilder::bucket) / [`set_bucket(Option<String>)`](crate::operation::get_bucket_request_payment::builders::GetBucketRequestPaymentFluentBuilder::set_bucket):<br>required: **true**<br><p>The name of the bucket for which to get the payment request configuration</p><br>
    ///   - [`expected_bucket_owner(impl Into<String>)`](crate::operation::get_bucket_request_payment::builders::GetBucketRequestPaymentFluentBuilder::expected_bucket_owner) / [`set_expected_bucket_owner(Option<String>)`](crate::operation::get_bucket_request_payment::builders::GetBucketRequestPaymentFluentBuilder::set_expected_bucket_owner):<br>required: **false**<br><p>The account ID of the expected bucket owner. If the account ID that you provide does not match the actual owner of the bucket, the request fails with the HTTP status code <code>403 Forbidden</code> (access denied).</p><br>
    /// - On success, responds with [`GetBucketRequestPaymentOutput`](crate::operation::get_bucket_request_payment::GetBucketRequestPaymentOutput) with field(s):
    ///   - [`payer(Option<Payer>)`](crate::operation::get_bucket_request_payment::GetBucketRequestPaymentOutput::payer): <p>Specifies who pays for the download and request fees.</p>
    /// - On failure, responds with [`SdkError<GetBucketRequestPaymentError>`](crate::operation::get_bucket_request_payment::GetBucketRequestPaymentError)
    pub fn get_bucket_request_payment(&self) -> crate::operation::get_bucket_request_payment::builders::GetBucketRequestPaymentFluentBuilder {
        crate::operation::get_bucket_request_payment::builders::GetBucketRequestPaymentFluentBuilder::new(self.handle.clone())
    }
}
