// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto2";

package spotify.playlist.cosmos.proto;

import "playlist_permission.proto";

option java_multiple_files = true;
option optimize_for = CODE_SIZE;
option java_package = "com.spotify.playlist.proto";

message SetMemberPermissionRequest {
    optional string playlist_uri = 1;
    optional string username = 2;
    optional playlist_permission.proto.PermissionLevel permission_level = 3;
    optional uint32 timeout_ms = 4;
}
