// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`PutBucketNotificationConfiguration`](crate::operation::put_bucket_notification_configuration::builders::PutBucketNotificationConfigurationFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`bucket(impl Into<String>)`](crate::operation::put_bucket_notification_configuration::builders::PutBucketNotificationConfigurationFluentBuilder::bucket) / [`set_bucket(Option<String>)`](crate::operation::put_bucket_notification_configuration::builders::PutBucketNotificationConfigurationFluentBuilder::set_bucket):<br>required: **true**<br><p>The name of the bucket.</p><br>
    ///   - [`notification_configuration(NotificationConfiguration)`](crate::operation::put_bucket_notification_configuration::builders::PutBucketNotificationConfigurationFluentBuilder::notification_configuration) / [`set_notification_configuration(Option<NotificationConfiguration>)`](crate::operation::put_bucket_notification_configuration::builders::PutBucketNotificationConfigurationFluentBuilder::set_notification_configuration):<br>required: **true**<br><p>A container for specifying the notification configuration of the bucket. If this element is empty, notifications are turned off for the bucket.</p><br>
    ///   - [`expected_bucket_owner(impl Into<String>)`](crate::operation::put_bucket_notification_configuration::builders::PutBucketNotificationConfigurationFluentBuilder::expected_bucket_owner) / [`set_expected_bucket_owner(Option<String>)`](crate::operation::put_bucket_notification_configuration::builders::PutBucketNotificationConfigurationFluentBuilder::set_expected_bucket_owner):<br>required: **false**<br><p>The account ID of the expected bucket owner. If the account ID that you provide does not match the actual owner of the bucket, the request fails with the HTTP status code <code>403 Forbidden</code> (access denied).</p><br>
    ///   - [`skip_destination_validation(bool)`](crate::operation::put_bucket_notification_configuration::builders::PutBucketNotificationConfigurationFluentBuilder::skip_destination_validation) / [`set_skip_destination_validation(Option<bool>)`](crate::operation::put_bucket_notification_configuration::builders::PutBucketNotificationConfigurationFluentBuilder::set_skip_destination_validation):<br>required: **false**<br><p>Skips validation of Amazon SQS, Amazon SNS, and Lambda destinations. True or false value.</p><br>
    /// - On success, responds with [`PutBucketNotificationConfigurationOutput`](crate::operation::put_bucket_notification_configuration::PutBucketNotificationConfigurationOutput)
    /// - On failure, responds with [`SdkError<PutBucketNotificationConfigurationError>`](crate::operation::put_bucket_notification_configuration::PutBucketNotificationConfigurationError)
    pub fn put_bucket_notification_configuration(
        &self,
    ) -> crate::operation::put_bucket_notification_configuration::builders::PutBucketNotificationConfigurationFluentBuilder {
        crate::operation::put_bucket_notification_configuration::builders::PutBucketNotificationConfigurationFluentBuilder::new(self.handle.clone())
    }
}
