// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto2";

package spotify.player.proto;

import "context_player_options.proto";
import "player_license.proto";

option optimize_for = CODE_SIZE;

message GlobalNode {
    optional ContextPlayerOptions options = 1;
    optional PlayerLicense license = 2;
    map<string, string> configuration = 3;
}
