// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto2";

package spotify.player.proto;

import "context_player_options.proto";
import "player_license.proto";
import "skip_to_track.proto";

option optimize_for = CODE_SIZE;

message PreparePlayOptions {
    optional ContextPlayerOptionOverrides player_options_override = 1;
    optional PlayerLicense license = 2;
    map<string, string> configuration_override = 3;
    optional string playback_id = 4;
    optional bool always_play_something = 5;
    optional SkipToTrack skip_to_track = 6;
    optional int64 seek_to = 7;
    optional bool initially_paused = 8;
    optional bool system_initiated = 9;
    repeated string suppressions = 10;
    optional PrefetchLevel prefetch_level = 11;
    optional string session_id = 12;
    optional AudioStream audio_stream = 13;
}

enum PrefetchLevel {
    kNone = 0;
    kMedia = 1;
}

enum AudioStream {
    kDefault = 0;
    kAlarm = 1;
}
