// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.descriptorextension;

option java_multiple_files = true;
option optimize_for = CODE_SIZE;
option java_package = "com.spotify.descriptorextension.proto";

message ExtensionDescriptor {
    string text = 1;
    float weight = 2;
    repeated ExtensionDescriptorType types = 3;
}

message ExtensionDescriptorData {
    repeated ExtensionDescriptor descriptors = 1;
}

enum ExtensionDescriptorType {
    UNKNOWN = 0;
    GENRE = 1;
    MOOD = 2;
    ACTIVITY = 3;
    INSTRUMENT = 4;
    TIME = 5;
    ERA = 6;
    AESTHETIC = 7;
}
