// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto3";

package google.protobuf;

option csharp_namespace = "Google.Protobuf.WellKnownTypes";
option objc_class_prefix = "GPB";
option cc_enable_arenas = true;
option go_package = "google.golang.org/protobuf/types/known/wrapperspb";
option java_multiple_files = true;
option java_outer_classname = "WrappersProto";
option java_package = "com.google.protobuf";

message DoubleValue {
    double value = 1;
}

message FloatValue {
    float value = 1;
}

message Int64Value {
    int64 value = 1;
}

message UInt64Value {
    uint64 value = 1;
}

message Int32Value {
    int32 value = 1;
}

message UInt32Value {
    uint32 value = 1;
}

message BoolValue {
    bool value = 1;
}

message StringValue {
    string value = 1;
}

message BytesValue {
    bytes value = 1;
}
