// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`GetDataEndpoint`](crate::operation::get_data_endpoint::builders::GetDataEndpointFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`stream_name(impl Into<String>)`](crate::operation::get_data_endpoint::builders::GetDataEndpointFluentBuilder::stream_name) / [`set_stream_name(Option<String>)`](crate::operation::get_data_endpoint::builders::GetDataEndpointFluentBuilder::set_stream_name):<br>required: **false**<br><p>The name of the stream that you want to get the endpoint for. You must specify either this parameter or a <code>StreamARN</code> in the request.</p><br>
    ///   - [`stream_arn(impl Into<String>)`](crate::operation::get_data_endpoint::builders::GetDataEndpointFluentBuilder::stream_arn) / [`set_stream_arn(Option<String>)`](crate::operation::get_data_endpoint::builders::GetDataEndpointFluentBuilder::set_stream_arn):<br>required: **false**<br><p>The Amazon Resource Name (ARN) of the stream that you want to get the endpoint for. You must specify either this parameter or a <code>StreamName</code> in the request.</p><br>
    ///   - [`api_name(ApiName)`](crate::operation::get_data_endpoint::builders::GetDataEndpointFluentBuilder::api_name) / [`set_api_name(Option<ApiName>)`](crate::operation::get_data_endpoint::builders::GetDataEndpointFluentBuilder::set_api_name):<br>required: **true**<br><p>The name of the API action for which to get an endpoint.</p><br>
    /// - On success, responds with [`GetDataEndpointOutput`](crate::operation::get_data_endpoint::GetDataEndpointOutput) with field(s):
    ///   - [`data_endpoint(Option<String>)`](crate::operation::get_data_endpoint::GetDataEndpointOutput::data_endpoint): <p>The endpoint value. To read data from the stream or to write data to it, specify this endpoint in your application.</p>
    /// - On failure, responds with [`SdkError<GetDataEndpointError>`](crate::operation::get_data_endpoint::GetDataEndpointError)
    pub fn get_data_endpoint(&self) -> crate::operation::get_data_endpoint::builders::GetDataEndpointFluentBuilder {
        crate::operation::get_data_endpoint::builders::GetDataEndpointFluentBuilder::new(self.handle.clone())
    }
}
