// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto3";

package spotify.local_sync_state.proto;

option optimize_for = CODE_SIZE;
option java_package = "com.spotify.local_sync_state.proto";

message State {
    string safe_secret = 1;
}
