// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.cosmos_util.proto;

import "extension_kind.proto";

option java_multiple_files = true;
option optimize_for = CODE_SIZE;
option java_package = "com.spotify.cosmos.util.policy.proto";

message ShowDecorationPolicy {
    bool link = 1;
    bool name = 2;
    bool description = 3;
    bool popularity = 4;
    bool publisher = 5;
    bool language = 6;
    bool is_explicit = 7;
    bool covers = 8;
    bool num_episodes = 9;
    bool consumption_order = 10;
    bool media_type_enum = 11;
    bool copyrights = 12;
    bool trailer_uri = 13;
    bool is_music_and_talk = 14;
    bool access_info = 15;
    repeated extendedmetadata.ExtensionKind extension = 16;
    bool is_book = 17;
}

message ShowPlayedStateDecorationPolicy {
    bool latest_played_episode_link = 1;
}
