// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.stream_reporting_esperanto.proto;

option objc_class_prefix = "ESP";
option java_package = "com.spotify.stream_reporting_esperanto.proto";

enum StreamingRule {
    STREAMING_RULE_NONE = 0;
    STREAMING_RULE_DMCA_RADIO = 1;
    STREAMING_RULE_PREVIEW = 2;
    STREAMING_RULE_WIFI = 3;
    STREAMING_RULE_SHUFFLE_MODE = 4;
}
