/*
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0
 */

//! Stalled stream protection configuration

// Re-export from aws-smithy-types
pub use aws_smithy_runtime_api::client::stalled_stream_protection::StalledStreamProtectionConfig;
