// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto2";

package spotify.player.proto.state_restore;

option optimize_for = CODE_SIZE;

message BehaviorMetadataRules {
    repeated string page_instance_ids = 1;
    repeated string interaction_ids = 2;
}
