// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.playlist_esperanto.proto;

import "contains_request.proto";
import "response_status.proto";

option objc_class_prefix = "ESP";
option java_multiple_files = true;
option optimize_for = CODE_SIZE;
option java_package = "spotify.playlist.esperanto.proto";

message PlaylistContainsRequest {
    string uri = 1;
    playlist.cosmos.proto.ContainsRequest request = 2;
}

message PlaylistContainsResponse {
    ResponseStatus status = 1;
    playlist.cosmos.proto.ContainsResponse response = 2;
}
