// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto2";

package spotify.playlist.cosmos.proto;

import "policy/playlist_decoration_policy.proto";

option java_multiple_files = true;
option optimize_for = CODE_SIZE;
option java_package = "com.spotify.playlist.policy.proto";

message RootlistPlaylistDecorationPolicy {
    optional bool add_time = 1;
    optional PlaylistDecorationPolicy playlist = 2;
    optional bool group_label = 3;
}
