// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto3";

package spotify.player.esperanto.proto;

import "es_context_page.proto";
import "es_restrictions.proto";

option objc_class_prefix = "ESP";
option optimize_for = CODE_SIZE;
option java_package = "com.spotify.player.esperanto.proto";

message Context {
    repeated ContextPage pages = 1;
    map<string, string> metadata = 2;
    string uri = 3;
    string url = 4;
    bool is_loaded = 5;
    Restrictions restrictions = 6;
}
