// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.your_library.proto;

option optimize_for = CODE_SIZE;

message PinRequest {
    string uri = 1;
}

message PinResponse {
    PinStatus status = 1;
    enum PinStatus {
        UNKNOWN = 0;
        PINNED = 1;
        NOT_PINNED = 2;
    }
    
    bool has_maximum_pinned_items = 2;
    int32 maximum_pinned_items = 3;
    string error = 99;
}

message PinItem {
    string uri = 1;
    bool in_library = 2;
}

message PinList {
    repeated PinItem item = 1;
    string error = 99;
}
