// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto2";

package spotify.player.proto.state_restore;

option optimize_for = CODE_SIZE;

message ContextAndPage {
    required string context_uri = 1;
    required string fallback_page_url = 2;
}

message MftFallbackPageHistory {
    repeated ContextAndPage context_to_fallback_page = 1;
}
