// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
impl super::Client {
    /// Constructs a fluent builder for the [`DeleteSignalingChannel`](crate::operation::delete_signaling_channel::builders::DeleteSignalingChannelFluentBuilder) operation.
    ///
    /// - The fluent builder is configurable:
    ///   - [`channel_arn(impl Into<String>)`](crate::operation::delete_signaling_channel::builders::DeleteSignalingChannelFluentBuilder::channel_arn) / [`set_channel_arn(Option<String>)`](crate::operation::delete_signaling_channel::builders::DeleteSignalingChannelFluentBuilder::set_channel_arn):<br>required: **true**<br><p>The Amazon Resource Name (ARN) of the signaling channel that you want to delete.</p><br>
    ///   - [`current_version(impl Into<String>)`](crate::operation::delete_signaling_channel::builders::DeleteSignalingChannelFluentBuilder::current_version) / [`set_current_version(Option<String>)`](crate::operation::delete_signaling_channel::builders::DeleteSignalingChannelFluentBuilder::set_current_version):<br>required: **false**<br><p>The current version of the signaling channel that you want to delete. You can obtain the current version by invoking the <code>DescribeSignalingChannel</code> or <code>ListSignalingChannels</code> API operations.</p><br>
    /// - On success, responds with [`DeleteSignalingChannelOutput`](crate::operation::delete_signaling_channel::DeleteSignalingChannelOutput)
    /// - On failure, responds with [`SdkError<DeleteSignalingChannelError>`](crate::operation::delete_signaling_channel::DeleteSignalingChannelError)
    pub fn delete_signaling_channel(&self) -> crate::operation::delete_signaling_channel::builders::DeleteSignalingChannelFluentBuilder {
        crate::operation::delete_signaling_channel::builders::DeleteSignalingChannelFluentBuilder::new(self.handle.clone())
    }
}
