// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto2";

package spotify.event_sender.proto;

option optimize_for = CODE_SIZE;

message RemainingSkips {
    optional string interaction_id = 1;
    optional int32 remaining_skips_before_skip = 2;
    optional int32 remaining_skips_after_skip = 3;
    repeated string interaction_ids = 4;
}
