// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto3";

package spotify.player.esperanto.proto;

import "es_context_track.proto";

option objc_class_prefix = "ESP";
option optimize_for = CODE_SIZE;
option java_package = "com.spotify.player.esperanto.proto";

message ProvidedTrack {
    ContextTrack context_track = 1;
    repeated string removed = 2;
    repeated string blocked = 3;
    string provider = 4;
}
