// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto3";

package spotify.collection.proto;

option optimize_for = CODE_SIZE;

message TagsSyncState {
    string uri = 1;
    bool sync_is_complete = 2;
}
