// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto3";

package spotify.player.esperanto.proto;

import "es_command_options.proto";
import "es_logging_params.proto";

option objc_class_prefix = "ESP";
option optimize_for = CODE_SIZE;
option java_package = "com.spotify.player.esperanto.proto";

message StopRequest {
    CommandOptions options = 1;
    
    Reason reason = 2;
    enum Reason {
        INTERACTIVE = 0;
        REMOTE_TRANSFER = 1;
        SHUTDOWN = 2;
    }
    
    LoggingParams logging_params = 3;
}
