// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto2";

package spotify.player.proto.state_restore;

option optimize_for = CODE_SIZE;

message ExplicitContentRules {
    required bool filter_explicit_content = 1;
    required bool filter_age_restricted_content = 2;
}
