// Code generated by software.amazon.smithy.rust.codegen.smithy-rs. DO NOT EDIT.
pub use ::aws_types::request_id::RequestId;

/// Types for the `CreateParallelData` operation.
pub mod create_parallel_data;

/// Types for the `DeleteParallelData` operation.
pub mod delete_parallel_data;

/// Types for the `DeleteTerminology` operation.
pub mod delete_terminology;

/// Types for the `DescribeTextTranslationJob` operation.
pub mod describe_text_translation_job;

/// Types for the `GetParallelData` operation.
pub mod get_parallel_data;

/// Types for the `GetTerminology` operation.
pub mod get_terminology;

/// Types for the `ImportTerminology` operation.
pub mod import_terminology;

/// Types for the `ListLanguages` operation.
pub mod list_languages;

/// Types for the `ListParallelData` operation.
pub mod list_parallel_data;

/// Types for the `ListTagsForResource` operation.
pub mod list_tags_for_resource;

/// Types for the `ListTerminologies` operation.
pub mod list_terminologies;

/// Types for the `ListTextTranslationJobs` operation.
pub mod list_text_translation_jobs;

/// Types for the `StartTextTranslationJob` operation.
pub mod start_text_translation_job;

/// Types for the `StopTextTranslationJob` operation.
pub mod stop_text_translation_job;

/// Types for the `TagResource` operation.
pub mod tag_resource;

/// Types for the `TranslateDocument` operation.
pub mod translate_document;

/// Types for the `TranslateText` operation.
pub mod translate_text;

/// Types for the `UntagResource` operation.
pub mod untag_resource;

/// Types for the `UpdateParallelData` operation.
pub mod update_parallel_data;
