// Extracted from: Spotify 1.1.61.583 (Windows)

syntax = "proto2";

package spotify.collection.proto.storage;

option optimize_for = CODE_SIZE;

message BanItem {
    required string item_uri = 1;
    required string context_uri = 2;
    required int64 timestamp = 3;
}

message Bans {
    repeated BanItem items = 1;
}
