// Extracted from: Spotify 1.1.73.517 (macOS)

syntax = "proto3";

package spotify.playlist_esperanto.proto;

import "response_status.proto";

option objc_class_prefix = "ESP";
option java_multiple_files = true;
option optimize_for = CODE_SIZE;
option java_package = "spotify.playlist.esperanto.proto";

message PlaylistSetMemberPermissionResponse {
    ResponseStatus status = 1;
}
